#include <iostream>
#include <set>
#include "Database.hpp"

bool read_name(const std::string& prompt,
      std::set<std::string>& already_named, std::string& name) {
   std::string s;
   for(;;) {
      std::cout << prompt;
      if (!std::getline(std::cin, s)) return false;
      auto it = already_named.find(s);
      if (it == already_named.end()) {
	 already_named.insert(s);
	 name = s;
	 return true;
      }
      std::cout << s << " was already named." << std::endl;
   }
}

ActorPtr read_actor(const Database& db) {
   std::string name;
   static std::set<std::string> named_actors;
   for(;;) {
      if (!read_name("Actor: ", named_actors, name)) return nullptr;
      ActorPtr actor = db.lookup_actor(name);
      if (actor) return actor;
      std::cout << name << " is not known." << std::endl;
   }
}

MoviePtr read_movie(const Database& db) {
   std::string name;
   static std::set<std::string> named_movies;
   for(;;) {
      if (!read_name("Movie: ", named_movies, name)) return nullptr;
      MoviePtr movie = db.lookup_movie(name);
      if (movie) return movie;
      std::cout << name << " is not known." << std::endl;
   }
}

bool associated(ActorPtr actor, MoviePtr movie) {
   if (!actor || !movie) return true;
   if (actor->lookup(movie->get_title())) return true;
   std::cout << "Sorry, but " << actor->get_name() <<
      " did not participate in '" << movie->get_title() << "'" << std::endl;
   return false;
}

int main(int argc, char** argv) {
   if (argc != 2) {
      std::cerr << "Usage: " << argv[0] << " movies" << std::endl; return 1;
   }
   Database db;
   if (!db.read_database(argv[1])) {
      std::cerr << "Unable to open " << argv[1] << " for reading" << std::endl;
      return 1;
   }
   std::cout << "*** Actors/Movies Game ***" << std::endl;
   std::string name; std::string title;
   ActorPtr actor = nullptr; MoviePtr movie = nullptr; std::size_t count = 0;
   for(;;) {
      actor = read_actor(db); if (!actor) break;
      if (!associated(actor, movie)) break;
      movie = read_movie(db); if (!movie) break;
      if (!associated(actor, movie)) break;
      ++count;
   }
   if (count > 1) {
      std::cout << "Well done, you have scored " << count <<
	 " points!" << std::endl;
   } else if (count == 1) {
      std::cout << "That was not very impressive!" << std::endl;
   } else {
      std::cout << "Disappointing!" << std::endl;
   }
}
